package com.framsticks.gui;

import com.framsticks.core.*;
import com.framsticks.observers.Endpoint;
import com.framsticks.observers.Observer;
import com.framsticks.util.Logging;
import com.framsticks.util.dispatching.Dispatcher;
import com.framsticks.util.dispatching.Future;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.lang.ArrayUtils;
import org.apache.log4j.Logger;

import javax.swing.*;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author Piotr Sniegowski
 */
public class Browser extends Observer implements Dispatcher {

	private static final Logger log = Logger.getLogger(Browser.class.getName());

	protected final Set<Frame> frames = new HashSet<Frame>();
	protected MainFrame mainFrame;
	public List<PanelProvider> panelProviders = new ArrayList<PanelProvider>();
	protected Dimension defaultFrameDimension;

	public void addFrame(Frame frame) {
		frames.add(frame);
	}

	public Browser() {
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		addPanelProvider(new StandardPanelProvider());
	}

	@Override
	public void configure(Configuration config) {
		super.configure(config);

		defaultFrameDimension = new Dimension(config.getInteger("size.width", 1000), config.getInteger("size.height", 500));

		for (String name : config.getStringArray("panel_providers")) {
			try {
				Class<?> c = Class.forName(name);
				if (ArrayUtils.indexOf(c.getInterfaces(), PanelProvider.class) == -1) {
					continue;
				}
				PanelProvider p = (PanelProvider)c.newInstance();
				addPanelProvider(p);
			} catch (Exception e) {
				log.error("failed to load PanelProvider " + name + ": " + e);
			}
		}

		// for (final String path : config.getStringArray("resolve_paths")) {
		//	invokeLater()
		//	autoResolvePath(path, new Future<Path>() {
		//		@Override
		//		public void result(Path p, Exception e) {
		//			Logging.log(log, "auto resolve path", path, e);
		//		}
		//	});
		// }
	}

	public void addPanelProvider(PanelProvider panelProvider) {
		log.debug("added panel provider of type: " + panelProvider.getClass().getCanonicalName());
		panelProviders.add(panelProvider);
	}

	public void autoResolvePath(final String path, final Future<Path> future) {
		final Instance i = endpoints.get("localhost").getInstance();
		i.invokeLater(new Runnable() {
			@Override
			public void run() {
				i.resolveAndFetch(path, new Future<Path>() {
					@Override
					public void result(final Path p, Exception e) {
						Logging.log(log, "auto resolve path", path, e);
						if (future != null) {
							future.result(p, e);
						}
						if (e == null) {
							mainFrame.invokeLater(new Runnable() {
								@Override
								public void run() {
									mainFrame.goTo(p);
								}
							});
						}
					}
				});
			}
		});
	}

	public void clear() {
		assert isActive();
		for (Frame f : frames) {
			f.clear();
		}
	}

	@Override
	public void run() {
		super.run();

		assert isActive();

		try {
			boolean found = false;
			// for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
			//	log.info("look and feel available: " + info.getName());
			//	if ("Nimbus".equals(info.getName())) {
			//		UIManager.setLookAndFeel(info.getClassName());
			//		found = true;
			//		break;
			//	}
			// }
			if (!found) {
				UIManager.setLookAndFeel("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
			}
		} catch (Exception ex) {
			log.warn("failed loading Look&Feel: ", ex);
		}

		javax.swing.JFrame.setDefaultLookAndFeelDecorated(true);

		mainFrame = new MainFrame(Browser.this);
		addFrame(mainFrame);

		for (Frame f : frames) {
			f.configure();
		}

		for (final Endpoint e : getEndpoints().values()) {
			e.invokeLater(new Runnable() {
				@Override
				public void run() {
					final Path p = e.getInstance().getRootPath();
					invokeLater(new Runnable() {
						@Override
						public void run() {
							mainFrame.addRootPath((BrowserEndpoint) e, p);
						}
					});
				}
			});
		}

		for (Frame f : frames) {
			f.setVisible(true);
		}

		// autoResolvePath("/simulator/genepools/groups/0", null);
		// autoResolvePath("/", null);
	}

	public void createTreeNodeForChild(final Path path) {
		assert !isActive();
		//assert instance.isActive();


/*
		final TreeNode parentTreeNode = (TreeNode) child.getParent().getUserObject();
		if (parentTreeNode == null) {
			Dispatching.invokeDispatch(this, manager, new Runnable() {
				@Override
				public void run() {
					createTreeNodeForChild(child);
				}
			});
			return;
		}
		log.debug(child.getClass().getSimpleName() + " created: " + child);


		invokeLater(new Runnable() {
			@Override
			public void run() {
				parentTreeNode.getOrCreateChildTreeNodeFor(child);
			}
		});
*/
	}


	@Override
	protected Endpoint createEndpoint() {
		return new BrowserEndpoint();
	}

	@Override
	public Dispatcher createDefaultDispatcher() {
		return SwingDispatcher.instance;
	}

	/**
	 * @return the mainFrame
	 */
	public MainFrame getMainFrame() {
		return mainFrame;
	}

}
